%% #!/bin/sh
%% exec ciao-shell $0 "$@" # -*- mode: ciao; -*-

%% Line above for CIAO, this one for SICStus
% exec prolog_shell3 $0 "$@"

%% SICStus-compatibility. Include for running on SICStus.
% :- [ciaocompat].

:- use_package(assertions).

%% CIAO-specific loading of libraries (comment out for SICStus).
:- use_module(library(format)).
:- use_module(library(ttyout)).
:- use_module(library(dec10_io)).

%----------------------------------------------------------------------
% Simple converter from bibtex file(s) to .pl format
%----------------------------------------------------------------------

:- ensure_loaded(bibparser).
:- ensure_loaded(support).
:- ensure_loaded(pp_papers_pl).

main(A) :- 
	bibfilenames(A,Out),
	!,
 	append(Out,"papers_auto.pl",OS),
 	name(O,OS),
 	format("[Processing ~w into ~w...]~n",[A,O]),ttyflush,
 	tell(O),
 	format("%% ----------------------------------------------------~n",[]),
 	format("%% WARNING: Do not edit this file~n",[]),
 	format("%% It has been generated automatically from file(s) ~w~n",[A]),
	format("%% ----------------------------------------------------~n",[]),
 	merge_files(A,Total),
 	seen,
	told,
	RTotal is Total - 1,
	format("[Done. ~w entries processed.]~n",[RTotal]),ttyflush.
main(A) :- 
	format("[ERROR: args ~w -- arguments must be .bib filenames.]",[A]).

bibfilenames([],[]).
bibfilenames([N|R],Out) :- 
	name(N,NS),
	append(NameS,".bib",NS),
	bibfilenames(R,Rest),	
	append(NameS,[0'_|Rest],Out).

merge_files(A,Total) :-
	readfiles(A,1,Total).

readfiles([],T,T).
readfiles([F|R],Count,Total) :-
 	format(user_error,"[Processing ~w ",[F]),ttyflush,
	see(F),	
	get_bib_entry(_,_), % Move to first "@" in file
	readfile(1,FTotal),
	seen,
 	format(user_error," ~w entries processed]~n",[FTotal]),ttyflush,
	NCount is Count + FTotal,
	readfiles(R,NCount,Total).

readfile(Count,Total) :-
	get_bib_entry(Chars,Terminator),
	Entry = [0'@|Chars],
 	(  bibparser(Paper,Entry,[]) 
 	-> format(user_error,".",[]),
	   pp_paper_pl(Paper)
 	;  format(user_error,
           "~n[ERROR: *** Syntax error or unsupported syntax in entry ~w:~n~s~n (IGNORED) *** ]~n", [Count,Entry]),
	   format(
           "~n%% [ERROR: *** Syntax error or unsupported syntax in entry ~w (IGNORED) ***]~n", [Count]),
	   flush_output
	),
	(  Terminator \== -1        % eof, 26 for C-Prolog and Dec10
   	-> NCount is Count + 1,
	   readfile(NCount,Total)
        ;  Count=Total).
	
%----------------------------------------------------------------------
:- comment(version_maintenance,dir(version)).

:- comment(version(0*4+0,1998/10/17,18:50*25+'MET DST'), "Now running
   natively in CIAO Prolog.  (Manuel Hermenegildo)").
%----------------------------------------------------------------------

